@@ -4,6 +4,8 @@ provided by the TU Munich in context of the research project iHEM
 (intelligent Home Energy Management).
 
 """
+
+import math
 from copy import copy
 from typing import Any, Dict
 
@@ -20,16 +22,24 @@ class Battery(Buffer):
     efficiency eta is modelled by fitting a polynomial model to data measured
     by TU Munich.
 
+    See::
+
+        Schimpe, M.; Piesch, C.; Hesse, H.C.; Paß, J.; Ritter, S.; Jossen, A.
+        Power Flow Distribution Strategy for Improved Power Electronics
+        Energy Efficiency in Battery Storage Systems: Development and
+        Implementation in a Utility-Scale System. Energies 2018, 11, 533.
+        https://doi.org/10.3390/en11030533
+
     You have to provide the two dictionaries *params* and *inits*.
     *params* provides the configuration parameters for the battery model and
     might look like this::
 
         {
-            'cap_kwh': 5,
-            'p_charge_max_kw': 1,
-            'p_discharge_max_kw': 1,
-            'soc_min_percent': 15,
-            'eta_pc': [-2.109566, 0.403556, 97.110770],
+            "cap_kwh": 5,
+            "p_charge_max_kw": 1,
+            "p_discharge_max_kw": 1,
+            "soc_min_percent": 15,
+            "eta_pc": [-2.109566, 0.403556, 97.110770],
         }
 
     Here *cap_kwh* is the electric capacity of the battery in kWh. *p_min_kw*
@@ -50,9 +60,7 @@ class Battery(Buffer):
     current value has no effect in the next simulation step, they are just set
     to None during initialization of the model::
 
-        {
-            'soc_percent': 50
-        }
+        {"soc_percent": 50}
 
     Attributes
     ----------
@@ -99,7 +107,20 @@ class Battery(Buffer):
             else:
                 nstate.p_kw = 0
         else:
-            nstate.p_kw = self.inputs.p_set_kw
+            # discharge
+            if self.inputs.p_set_kw * self.config.gsign > 0:
+                if self.config.p_discharge_max_kw > abs(self.inputs.p_set_kw):
+                    nstate.p_kw = self.inputs.p_set_kw
+                else:
+                    nstate.p_kw = self.config.p_discharge_max_kw
+            # charge
+            else:
+                if self.config.p_charge_max_kw > abs(self.inputs.p_set_kw):
+                    nstate.p_kw = self.inputs.p_set_kw
+                else:
+                    nstate.p_kw = self.config.p_charge_max_kw
+            # copy sign to allow all sign conventions
+            nstate.p_kw = math.copysign(nstate.p_kw, self.inputs.p_set_kw)
 
     def _calculate_efficiency(self, nstate: BatteryState):
         """Calculate efficiency using a second degree polynomial."""
@@ -131,12 +152,11 @@ class Battery(Buffer):
         else:
             # Will be fully discharged in this step
             nstate.p_kw = theoretical_energy_kwh / (
-                self.inputs.step_size / 3600
+                self.inputs.step_size / 3600 / nstate.eta_percent * -100
             )
             nstate._energy_kwh = (
                 self.config.soc_min_percent / 100 * self.config.cap_kwh
             )
-            nstate.p_kw *= nstate.eta_percent / -100
 
     def _charge(self, nstate: BatteryState):
         delta_energy_kwh = (
@@ -151,7 +171,6 @@ class Battery(Buffer):
         else:
             # Will be fully charged in this step
             nstate.p_kw = (self.config.cap_kwh - nstate._energy_kwh) / (
-                self.inputs.step_size / 3600
+                self.inputs.step_size / 3600 * nstate.eta_percent / 100
             )
             nstate._energy_kwh = self.config.cap_kwh
-            nstate.p_kw /= nstate.eta_percent * 100
