@@ -57,8 +57,8 @@ class HVACConfig(ModelConfig):
     def __init__(self, params):
         super().__init__(params)
 
-        self.pn_min_kw = 0
-        self.pn_max_kw = params.get("p_max_kw", 2)
+        self.p_min_kw = 0
+        self.p_max_kw = params.get("p_max_kw", 2)
         self.eta_percent = params.get("eta_percent", 200.0)
         self.l_m = params.get("l_m", 4)
         self.w_m = params.get("w_m", 5)
@@ -77,28 +77,10 @@ class HVACConfig(ModelConfig):
         self.thaw_factor = params.get("thaw_factor", 1.0)
         self.cool_factor = params.get("cool_factor", 1.0)
 
-        self._default_schedule = list()
-        for _ in range(8):
-            self._default_schedule.extend([80, 10, 10])
+        # Model follows cooling demand;
+        # Use ScheduleModel to set schedules
+        self.default_schedule = None
 
-    @property
-    def p_min_kw(self):
-        if self.psc:
-            return self.pn_min_kw * self.lsign
-        else:
-            return self.pn_max_kw * self.lsign
-
-    @property
-    def p_max_kw(self):
-        if self.psc:
-            return self.pn_max_kw * self.lsign
-        else:
-            return self.pn_min_kw * self.lsign
-
-    @property
-    def q_min_kvar(self):
-        return 0
-
-    @property
-    def q_max_kvar(self):
-        return 0
+        # self.default_schedule = list()
+        # for _ in range(8):
+        #     self.default_schedule.extend([80, 10, 10])
