@@ -1,7 +1,9 @@
 """This module contains the HVAC model."""
+
 from copy import copy, deepcopy
 
 import numpy as np
+
 from pysimmods.consumer.hvacsim.config import HVACConfig
 from pysimmods.consumer.hvacsim.inputs import HVACInputs
 from pysimmods.consumer.hvacsim.state import HVACState
@@ -129,11 +131,8 @@ class HVAC(Consumer):
             return super().set_percent(percentage)
 
     def get_default_setpoint(self, hour: int) -> float:
-
         test_state = deepcopy(self.state)
         self._check_constraints(test_state)
         return self._get_percent(
-            test_state.p_kw,
-            self.get_pn_min_kw(),
-            self.get_pn_max_kw(),
+            test_state.p_kw, self.get_pn_min_kw(), self.get_pn_max_kw()
         )
