@@ -38,7 +38,7 @@ class BiogasConfig(ModelConfig):
         attributes section. The key for each attribute is the same as
         the attribute name, e.g.,::
 
-             {'gas_m3_per_day': 100}
+             {"gas_m3_per_day": 100}
 
         to set the attribute 'gas_m3_per_day.
 
