@@ -742,30 +742,9 @@ def params_3g_1500kw():
         "has_heat_storage": False,
         "num_chps": 3,
         "chp0": {
-            "pn_stages_kw": [
-                125.0,
-                250.0,
-                375.0,
-                500.0,
-                625.0,
-                750.0,
-            ],
-            "eta_stages_percent": [
-                26.1,
-                27.0,
-                27.3,
-                28.0,
-                27.7,
-                27.5,
-            ],
-            "eta_th_stages_percent": [
-                50.0,
-                50.2,
-                50.5,
-                50.9,
-                51.0,
-                52.1,
-            ],
+            "pn_stages_kw": [125.0, 250.0, 375.0, 500.0, 625.0, 750.0],
+            "eta_stages_percent": [26.1, 27.0, 27.3, 28.0, 27.7, 27.5],
+            "eta_th_stages_percent": [50.0, 50.2, 50.5, 50.9, 51.0, 52.1],
             "restarts_per_day": 0,
             "active_min_s": 3 * 3_600,
             "active_max_s_per_day": 0,
