@@ -1,10 +1,10 @@
-"""This module contains a CHP model that is used by the biogas plant.
+"""This module contains a CHP model that is used by the biogas plant."""
 
-"""
 from copy import deepcopy
 from datetime import timedelta
 
 import numpy as np
+
 from pysimmods.generator.chpcngsim.config import CHPCNGConfig
 from pysimmods.generator.chpcngsim.inputs import CHPCNGInputs
 from pysimmods.generator.chpcngsim.state import CHPCNGState
@@ -89,7 +89,6 @@ class CHPCNG(Generator):
         return abs(setpoint)
 
     def _check_constraints(self, next_state):
-
         if not self.inputs.gas_critical:
             self._check_daily_incative_time(next_state)
             self._check_restarts(next_state)
