@@ -1,6 +1,7 @@
 """This module contains the python port of a CHP model developed in the
 student's project group POWDER.
 """
+
 from copy import copy
 
 from pysimmods.generator.chplpgsim.config import CHPLPGConfig
@@ -26,7 +27,7 @@ class CHPLPG(Generator):
     def __init__(self, params, inits):
         self.config = CHPLPGConfig(params)
         self.state = CHPLPGState(inits)
-        self.inputs = CHPLPGInputs()
+        self.inputs: CHPLPGInputs = CHPLPGInputs()
 
         self._storage = HeatStorage(params, inits)
 
@@ -35,6 +36,8 @@ class CHPLPG(Generator):
         next_state = copy(self.state)
         next_state.p_kw = self._get_setpoint()
 
+        self._check_inputs()
+
         # Operating state constraint is checked first to prevent
         # invalid user inputs. However, this has least priority and
         # changes due to other constraints are possible.
@@ -61,6 +64,10 @@ class CHPLPG(Generator):
         self.state = next_state
         self.inputs.reset()
 
+    def _check_inputs(self):
+        if self.inputs.e_th_demand_set_kwh is None:
+            self.inputs.e_th_demand_set_kwh = 0
+
     def _get_setpoint(self):
         setpoint = self.inputs.p_set_kw
 
