@@ -3,6 +3,7 @@ developed within the scope of the research project iHEM (intelligent
 Home Energy Management).
 
 """
+
 from pysimmods.generator.pvsim.config import PVConfig
 from pysimmods.generator.pvsim.inputs import PVInputs
 from pysimmods.generator.pvsim.state import PVState
@@ -34,10 +35,7 @@ class PhotovoltaicPowerPlant(Generator):
     The dict *params* provides the parameters for the PV plant model
     and should at least look like this::
 
-        {
-            'a_m2': 15.0,
-            'eta_percent': 25,
-        }
+        {"a_m2": 15.0, "eta_percent": 25}
 
     The parameter *a_m2* specifies the overall area of the PV modules
     in square meter. Additional parameters can be provided, see
@@ -47,16 +45,14 @@ class PhotovoltaicPowerPlant(Generator):
     The peak power of the PV plant depends directly on the size of the
     Surface and the efficiency and is defined as::
 
-        p_peak_kw = eta_percent/100 * a_m2
+        p_peak_kw = eta_percent / 100 * a_m2
 
     The dict *inits* provides the initial values for the state variables.
     The PV plant model has one state variable that must be specified when
     the model is initialized. It is *t_module_deg_celsius* (indicates the
     temperature of the PV modules in °C)::
 
-            {
-                't_module_deg_celsius': 25
-            }
+            {"t_module_deg_celsius": 25}
 
     Parameters
     ----------
