@@ -19,6 +19,8 @@ class PVSystemState(ModelState):
         See :attr:`~.ModelState.p_kw`
     q_kvar : float
         See :attr:`~.ModelState.q_kvar`
+    p_possible_max_kw: float
+        Power output of the PV module before inverter gets active.
 
     """
 
@@ -26,3 +28,4 @@ class PVSystemState(ModelState):
         super().__init__(inits)
 
         self.t_module_deg_celsius = inits["pv"]["t_module_deg_celsius"]
+        self.p_possible_max_kw = 0
