@@ -191,3 +191,9 @@ class Buffer(Model):
 
     def get_p_kw(self) -> float:
         return self.state.p_kw * self.config.lsign
+
+    def get_qn_min_kvar(self) -> float:
+        return 0
+
+    def get_qn_max_kvar(self) -> float:
+        return 0
