@@ -19,3 +19,9 @@ class Generator(Model):
 
     def get_p_kw(self) -> float:
         return self.state.p_kw * self.config.gsign
+
+    def get_qn_min_kvar(self) -> float:
+        return 0
+
+    def get_qn_max_kvar(self) -> float:
+        return 0
