@@ -1,8 +1,9 @@
 """This module contains the base class for all pysimmods models."""
+
 import copy
 from abc import ABC, abstractmethod
 from datetime import datetime
-from typing import List, Union
+from typing import List, Optional, Union
 
 from .config import ModelConfig
 from .inputs import ModelInputs
@@ -113,6 +114,10 @@ class Model(ABC):
     def get_p_kw(self) -> float:
         """Return the current active power output of the model."""
 
+    def get_p_set_kw(self) -> Optional[float]:
+        """Return the current setpoint for active power of the model."""
+        return self.inputs.p_set_kw
+
     def get_pn_max_kw(self) -> float:
         """Maximum nominal active power output of the model.
 
@@ -134,6 +139,12 @@ class Model(ABC):
         """
         return self.config.p_min_kw
 
+    def set_cos_phi(self, cos_phi: float) -> None:
+        self.inputs.cos_phi = max(0, min(1, cos_phi))
+
+    def get_cos_phi(self) -> Optional[float]:
+        return self.state.cos_phi
+
     def set_percent(self, percentage: float) -> None:
         """Convenience function to set the desired output of the model.
 
@@ -224,7 +235,7 @@ class Model(ABC):
         """
         raise NotImplementedError()
 
-    def set_q_kvar(self, q_kvar) -> None:
+    def set_q_kvar(self, q_kvar: Optional[float]) -> None:
         """Set the target value for reactive power for the model's next
         step.
 
@@ -232,7 +243,10 @@ class Model(ABC):
         q_max_kvar.
 
         """
-        raise NotImplementedError()
+        self.inputs.q_set_kvar = q_kvar
+
+    def get_q_set_kvar(self) -> Optional[float]:
+        return self.inputs.q_set_kvar
 
     def get_q_kvar(self) -> float:
         """Return the current reactive power output of the model."""
@@ -262,19 +276,31 @@ class Model(ABC):
         """
         raise NotImplementedError()
 
-    def get_default_schedule(self) -> List[float]:
-        """Return the default schedule.
+    def get_default_p_schedule(self) -> List[float]:
+        """Return the default schedule for active power.
 
         Returns
         -------
         list
-            A *list* containing a default schedule with percentage
-            power for each hour of the day (i.e.,
-            len(default_schedule) == 24). This is used if no other *p*
-            or *q* input is provided.
+            A *list* containing a default schedule for active power for
+            each hour of the day (i.e., len(default_schedule) == 24).
+            This is used if no other *p* input is provided.
 
         """
-        return self.config.default_schedule
+        return self.config.default_p_schedule
+
+    def get_default_q_schedule(self) -> List[float]:
+        """Return the default schedule for reactive power.
+
+        Returns
+        -------
+        list
+            A *list* containing a default schedule for reactive power
+            for each hour of the day (i.e., len(default_q_schedule) ==
+            24). This is used if no other *q* input is provided.
+
+        """
+        return self.config.default_p_schedule
 
     def _get_percent(self, val, val_min, val_max):
         if val > 0:
@@ -295,7 +321,20 @@ class Model(ABC):
             # Unit is off or does not follow primary purpose
             return 0
 
-    def get_default_setpoint(self, hour: int) -> float:
+    def get_default_p_set(self, hour: int) -> Optional[float]:
         """Return the value of the default schedule at *hour*"""
         hour = max(0, min(23, hour))
-        return self.config.default_schedule[hour]
+        try:
+            return self.config.default_p_schedule[hour]
+        except TypeError:
+            # There is no schedule
+            return None
+
+    def get_default_q_set(self, hour: int) -> Optional[float]:
+        """Return the value of the default schedule at *hour*"""
+        hour = max(0, min(23, hour))
+        try:
+            return self.config.default_q_schedule[hour]
+        except TypeError:
+            # There is no schedule
+            return None
