@@ -3,10 +3,10 @@ flexiblity model, which is a wrapper for all models of the
 pysimmods package.
 
 """
-from __future__ import annotations
-import json
 
-import sys
+from __future__ import annotations
+
+import json
 from datetime import datetime, timedelta, timezone
 from typing import Any, Dict, List, Union
 
@@ -16,6 +16,7 @@ import pandas as pd
 from midas.util.dict_util import tobool
 from midas.util.logging import set_and_init_logger
 from midas.util.runtime_config import RuntimeConfig
+
 from pysimmods.mosaik import LOG
 from pysimmods.other.flexibility.flexibility_model import FlexibilityModel
 from pysimmods.other.flexibility.forecast_model import ForecastModel
@@ -164,18 +165,12 @@ class FlexibilitySimulator(PysimmodsSimulator):
             eid = f"{model}-{self.num_models[model]}"
 
             if self.provide_flexibilities:
-                # FIXME
                 self.models[eid] = FlexibilityModel(
                     MODELS[model](params, inits),
-                    self.now_dt,
-                    step_size=self.step_size,
-                    forecast_horizon_hours=self.forecast_horizon_hours,
-                    flexibility_horizon_hours=self.flexibility_horizon_hours,
-                    num_schedules=self.num_schedules,
-                    seed=self.rng.randint(sys.maxsize),
                     unit=self.unit,
-                    use_decimal_percent=self.use_decimal_percent,
-                    priorize_schedule=self.prioritize_setpoint,
+                    prioritize_setpoint=self.prioritize_setpoint,
+                    forecast_horizon_hours=self.forecast_horizon_hours,
+                    seed=self.rng.randint(2**32 - 1),
                 )
 
             elif self.provide_forecasts:
@@ -261,7 +256,8 @@ class FlexibilitySimulator(PysimmodsSimulator):
 
             for attr in attrs:
                 if attr == "flexibilities":
-                    value = self.models[eid].flexibilities
+                    value = self._get_attr_flexibilities(eid)
+                    log_msg[attr] = json.loads(value)
 
                 elif attr == "schedule":
                     value = self._get_attr_schedule(eid)
@@ -278,6 +274,8 @@ class FlexibilitySimulator(PysimmodsSimulator):
 
         if not self.key_value_logs:
             LOG.debug("Gathered outputs: %s.", data)
+        else:
+            LOG.debug(data)
 
         return data
 
@@ -325,18 +323,26 @@ class FlexibilitySimulator(PysimmodsSimulator):
 
         return value
 
+    def _get_attr_flexibilities(self, eid: str) -> str:
+        dict_of_json = self.models[eid].flexibilities.to_json()
+        # value = json.dumps(dict_of_json)
+        return dict_of_json
+
     def _generate_flexibilities(self):
         if self.provide_flexibilities:
             for model in self.models.values():
                 model.generate_schedules(
-                    self.now_dt.strftime(GER),
+                    (
+                        self.now_dt
+                        + timedelta(hours=self.planning_horizon_hours)
+                    ).strftime(GER),
                     self.flexibility_horizon_hours,
                     self.num_schedules,
                 )
 
 
 def deserialize_schedule(
-    schedule: Union[pd.DataFrame, Dict[str, Any], str]
+    schedule: Union[pd.DataFrame, Dict[str, Any], str],
 ) -> pd.DataFrame:
     """Convert the schedule provided by mosaik to DataFrame"""
 
