@@ -1,15 +1,16 @@
 """This module contains the *mosaik_api* meta definition for pysimmods
 models.
 """
+
 from pysimmods.buffer.batterysim import Battery
 from pysimmods.consumer.hvacsim import HVAC
+from pysimmods.generator import WindPowerPlantSystem
 from pysimmods.generator.biogassim import BiogasPlant
 from pysimmods.generator.chplpgsystemsim import CHPLPGSystem
 from pysimmods.generator.dieselsim import DieselGenerator
 from pysimmods.generator.pvsystemsim import PVPlantSystem
 
 _SHARED_ATTRS = [
-    "set_percent",
     "p_set_mw",
     "p_set_kw",
     "p_mw",
@@ -40,6 +41,7 @@ META = {
                 "dh_w_per_m2",
                 "s_module_w_per_m2",
                 "inverter_inductive",
+                "p_possible_max_mw",
             ]
             + _SHARED_ATTRS,
         },
@@ -57,7 +59,7 @@ META = {
         "HVAC": {
             "public": True,
             "params": ["params", "inits"],
-            "attrs": ["t_air_deg_celsius", "local_time"] + _SHARED_ATTRS,
+            "attrs": ["t_air_deg_celsius"] + _SHARED_ATTRS,
         },
         "Biogas": {
             "public": True,
@@ -69,6 +71,18 @@ META = {
             "params": ["params", "inits"],
             "attrs": _SHARED_ATTRS,
         },
+        "WindTurbine": {
+            "public": True,
+            "params": ["params", "inits"],
+            "attrs": [
+                "wind_v_m_per_s",
+                "t_air_deg_celsius",
+                "air_pressure_hpa",
+                "inverter_inductive",
+                "p_possible_max_mw",
+            ]
+            + _SHARED_ATTRS,
+        },
     },
 }
 
@@ -79,4 +93,5 @@ MODELS = {
     "CHP": CHPLPGSystem,
     "DieselGenerator": DieselGenerator,
     "Biogas": BiogasPlant,
+    "WindTurbine": WindPowerPlantSystem,
 }
