@@ -1,16 +1,19 @@
 """MIDAS upgrade module for the weather data simulator."""
+
 import logging
+from importlib import import_module
 
 import pandas as pd
-from midas.util.upgrade_module import UpgradeModule
+from midas.scenario.upgrade_module import UpgradeModule
+from mosaik.exceptions import ScenarioError
+
 from pysimmods.buffer.batterysim.presets import battery_preset
 from pysimmods.consumer.hvacsim.presets import hvac_preset
 from pysimmods.generator.biogassim.presets import biogas_preset
 from pysimmods.generator.chplpgsystemsim.presets import chp_preset
 from pysimmods.generator.dieselsim.presets import diesel_presets
 from pysimmods.generator.pvsystemsim.presets import pv_preset
-
-from mosaik.exceptions import ScenarioError
+from pysimmods.generator.windsystemsim.presets import wind_presets
 
 from .analysis.analysis import analyze
 
@@ -45,7 +48,12 @@ class PysimmodsModule(UpgradeModule):
                 "Photovoltaic",
                 "sgen",
                 ["bh_w_per_m2", "dh_w_per_m2", "t_air_deg_celsius"],
-                ["p_mw", "q_mvar", "t_module_deg_celsius"],
+                [
+                    "p_mw",
+                    "q_mvar",
+                    "t_module_deg_celsius",
+                    "p_possible_max_mw",
+                ],
             ),
             "HVAC": (
                 "HVAC",
@@ -59,12 +67,7 @@ class PysimmodsModule(UpgradeModule):
                 ["day_avg_t_air_deg_celsius"],
                 ["p_mw", "q_mvar"],
             ),
-            "DIESEL": (
-                "DieselGenerator",
-                "sgen",
-                [],
-                ["p_mw", "q_mvar"],
-            ),
+            "DIESEL": ("DieselGenerator", "sgen", [], ["p_mw", "q_mvar"]),
             "BAT": (
                 "Battery",
                 "storage",
@@ -72,9 +75,16 @@ class PysimmodsModule(UpgradeModule):
                 ["p_mw", "q_mvar", "soc_percent"],
             ),
             "Biogas": ("Biogas", "sgen", [], ["p_mw", "q_mvar"]),
+            "Wind": (
+                "WindTurbine",
+                "sgen",
+                ["t_air_deg_celsius", "wind_v_m_per_s", "air_pressure_hpa"],
+                ["p_mw", "q_mvar", "p_possible_max_mw"],
+            ),
         }
-        self.sensors = list()
-        self.actuators = list()
+        self.sensors = []
+        self.actuators = []
+        self._models_started = {}
 
     def check_module_params(self, module_params):
         """Check the module params and provide default values."""
@@ -84,8 +94,7 @@ class PysimmodsModule(UpgradeModule):
         module_params.setdefault("q_control", "prioritize_p")
         module_params.setdefault("inverter_mode", "inductive")
         module_params.setdefault(
-            "forecast_horizon_hours",
-            self.scenario.base.forecast_horizon_hours,
+            "forecast_horizon_hours", self.scenario.base.forecast_horizon_hours
         )
         module_params.setdefault("pv_is_static_t_module", False)
         module_params.setdefault("provide_flexibilities", False)
@@ -108,6 +117,7 @@ class PysimmodsModule(UpgradeModule):
         module_params.setdefault(
             "enable_schedules", module_params["provide_forecasts"]
         )
+        module_params.setdefault("pv_send_p_possible_max_mw_to_grid", False)
 
     def check_sim_params(self, module_params, **kwargs):
         """Check the params for a certain simulator instance."""
@@ -122,12 +132,14 @@ class PysimmodsModule(UpgradeModule):
             "pv_is_static_t_module", module_params["pv_is_static_t_module"]
         )
         self.sim_params.setdefault(
-            "forecast_horizon_hours",
-            module_params["forecast_horizon_hours"],
+            "pv_send_p_possible_max_mw_to_grid",
+            module_params["pv_send_p_possible_max_mw_to_grid"],
         )
         self.sim_params.setdefault(
-            "provide_flexibilities",
-            module_params["provide_flexibilities"],
+            "forecast_horizon_hours", module_params["forecast_horizon_hours"]
+        )
+        self.sim_params.setdefault(
+            "provide_flexibilities", module_params["provide_flexibilities"]
         )
         self.sim_params.setdefault(
             "flexibility_horizon_hours",
@@ -138,16 +150,14 @@ class PysimmodsModule(UpgradeModule):
             module_params["flexibility_horizon_start_hours"],
         )
         self.sim_params.setdefault(
-            "flexibility_frequency",
-            module_params["flexibility_frequency"],
+            "flexibility_frequency", module_params["flexibility_frequency"]
         )
         self.sim_params.setdefault(
             "num_schedules", module_params["num_schedules"]
         )
         self.sim_params.setdefault("unit", module_params["unit"])
         self.sim_params.setdefault(
-            "use_decimal_percent",
-            module_params["use_decimal_percent"],
+            "use_decimal_percent", module_params["use_decimal_percent"]
         )
         self.sim_params.setdefault(
             "prioritize_setpoint", module_params["prioritize_setpoint"]
@@ -182,8 +192,14 @@ class PysimmodsModule(UpgradeModule):
             elif self.sim_params["cmd"] == "cmd":
                 self.sim_params["import_str"] = self.flex_cmd_str
 
+        self.sensors = []
+        self.actuators = []
+        self._models_started = {}
+
     def start_models(self):
         """Start all models defined in the mapping of a certain simulator."""
+
+        self._models_started = {}
         mapping_key = "peak_mapping"
         if not self.sim_params.get(mapping_key, {}):
             if self.sim_params.get("mapping", {}):
@@ -195,10 +211,29 @@ class PysimmodsModule(UpgradeModule):
             # No mapping configured
             return
 
+        if ":" in self.default_import_str:
+            mod, clazz = self.default_import_str.split(":")
+        else:
+            mod, clazz = self.default_import_str.rsplit(".", 1)
+        mod = import_module(mod)
+
+        sim_dummy = getattr(mod, clazz)()
+        sim_dummy.init(self.sid, **self.sim_params)
+
         eid_mapping = self.scenario.create_shared_mapping(
             self, self.sim_params["grid_name"], "eid"
         )
-        high = "1.0" if self.sim_params["use_decimal_percent"] else "100.0"
+        try:
+            pg_mapping = self.scenario.get_powergrid_mappings(
+                self.sim_params["grid_name"]
+            )
+        except Exception:
+            LOG.exception(
+                "Please update midas-mosaik or powergrid mapping will not "
+                "work."
+            )
+            pg_mapping = {}
+        # high = "1.0" if self.sim_params["use_decimal_percent"] else "100.0"
         model_ctrs = {}
 
         for model, info in self.models.items():
@@ -214,7 +249,9 @@ class PysimmodsModule(UpgradeModule):
                         model_key, model, p_peak_mw
                     )
                     sn_mva = params["params"].get("sn_mva", p_peak_mw)
-                    full_id = self.start_model(model_key, info[0], params)
+                    self.start_model(model_key, info[0], params)
+                    entity = sim_dummy.create(1, info[0], **params)[0]
+                    full_id = f"{self.sid}.{entity['eid']}"
                     p_min, p_max, q_min, q_max = get_space(
                         model.lower(), params, p_peak_mw
                     )
@@ -225,6 +262,12 @@ class PysimmodsModule(UpgradeModule):
                         "type": info[1],
                         "sn_mva": sn_mva,
                     }
+                    bus_mapping = pg_mapping.setdefault(bus, {})
+                    module_mapping = bus_mapping.setdefault(
+                        self.module_name, {}
+                    )
+                    model_mapping = module_mapping.setdefault(model, [])
+                    model_mapping.append(model_key)
                     if p_min != p_max:
                         self.actuators.append(
                             create_actuator(full_id, "p_set_mw", p_min, p_max)
@@ -235,21 +278,31 @@ class PysimmodsModule(UpgradeModule):
                                 full_id, "q_set_mvar", q_min, q_max
                             )
                         )
-                    self.actuators.append(
-                        create_actuator(full_id, "set_percent", 0, high)
-                    )
+                    # self.actuators.append(
+                    #     create_actuator(full_id, "set_percent", 0, high)
+                    # )
                     for attr in self.models[model][3]:
                         self.sensors.append(create_sensor(full_id, attr))
 
+                    self._models_started[model_key] = {
+                        "bus": bus,
+                        "type": info[1],
+                        "model": info[0],
+                        "alias": model,
+                        "inputs": info[2],
+                        "outputs": info[3],
+                        "model_ctr": model_ctrs[model],
+                    }
                     model_ctrs[model] += 1
 
     def connect(self):
         """Connect the models to existing other models."""
 
-        for model, info in self.models.items():
-            self._connect_to_weather(model, info)
+        for model_key, info in self._models_started.items():
+            # for model, info in self.models.items():
+            self._connect_to_weather(model_key, info)
             try:
-                self._connect_to_grid(model, info)
+                self._connect_to_grid(model_key, info)
             except KeyError:
                 LOG.warning(
                     "No grid for %s found. Will output only to database "
@@ -257,89 +310,93 @@ class PysimmodsModule(UpgradeModule):
                     self.sim_params["sim_name"],
                 )
 
-    def _connect_to_weather(self, model, info):
-        mapping_key = "peak_mapping"
+    def _connect_to_weather(self, model_key, info):
+        # mapping_key = "peak_mapping"
         wpm_key = "weather_provider_mapping"
         wfm_key = "weather_forecast_mapping"
+        model_ctr = info["model_ctr"]
 
         if not self.sim_params.get(wpm_key, {}):
             self.sim_params[wpm_key] = self.create_default_wpm()
         self.sim_params.setdefault(wfm_key, {})
+        weather_key = self.get_weather_model(info["alias"], model_ctr)
 
-        model_ctrs = {}
+        # model_ctrs = {}
 
-        for bus, entities in self.sim_params[mapping_key].items():
-            for name, _ in entities:
-                if model != name:
-                    continue
+        # for bus, entities in self.sim_params[mapping_key].items():
+        #     for name, _ in entities:
+        #         if model != name:
+        #             continue
 
-                model_ctrs.setdefault(model, 0)
-                model_key = self.scenario.generate_model_key(
-                    self, model.lower(), bus, model_ctrs[model]
-                )
-                weather_key = self.get_weather_model(model, model_ctrs[model])
-                try:
-                    self.connect_entities(weather_key, model_key, info[2])
-                except KeyError:
-                    LOG.warning(
-                        "Weather mapping missing for %s. The simulation"
-                        " will fail if the model requires weather input",
-                        model,
-                    )
+        # model_ctrs.setdefault(model, 0)
+        # model_key = self.scenario.generate_model_key(
+        #     self, model.lower(), bus, model_ctrs[model]
+        # )
 
-                weather_key = self.get_weather_model(
-                    model, model_ctrs[model], True
-                )
-                if weather_key is not None:
-                    fc_attrs = [f"forecast_{a}" for a in info[2]]
-                    self.connect_entities(weather_key, model_key, fc_attrs)
+        try:
+            self.connect_entities(weather_key, model_key, info["inputs"])
+        except KeyError:
+            LOG.warning(
+                "Weather mapping missing for %s. The simulation"
+                " will fail if the model requires weather input",
+                info["model"],
+            )
 
-                model_ctrs[model] += 1
+        weather_key = self.get_weather_model(info["alias"], model_ctr, True)
+        if weather_key is not None:
+            fc_attrs = [f"forecast_{a}" for a in info[2]]
+            self.connect_entities(weather_key, model_key, fc_attrs)
 
-    def _connect_to_grid(self, model, info):
-        mapping_key = "peak_mapping"
-        model_ctrs = {}
+        # model_ctrs[model] += 1
 
-        for bus, entities in self.sim_params[mapping_key].items():
-            for name, _ in entities:
-                if model != name:
-                    continue
+    def _connect_to_grid(self, model_key, info):
+        # mapping_key = "peak_mapping"
+        # model_ctrs = {}
+        # model_ctr = info["model_ctr"]
+        # for bus, entities in self.sim_params[mapping_key].items():
+        #     for name, _ in entities:
+        #         if model != name:
+        #             continue
 
-                model_ctrs.setdefault(model, 0)
-                model_key = self.scenario.generate_model_key(
-                    self, model.lower(), bus, model_ctrs[model]
-                )
-                grid_entity_key = None
-                try:
-                    grid_entity_key = self.get_grid_entity(info[1], bus)
-                    self.connect_entities(
-                        model_key, grid_entity_key, info[3][:2]
-                    )
-                except ScenarioError as e:
-                    LOG.warning(
-                        "Encountered scenario error while connecting %s"
-                        "to grid entity %s: %s",
-                        model_key,
-                        grid_entity_key,
-                        e,
-                    )
-                except KeyError as e:
-                    LOG.warning(
-                        "Entity missing for %s: %s or for grid: %s (%s).",
-                        model,
-                        model_key,
-                        grid_entity_key,
-                        e,
-                    )
-                except ValueError as e:
-                    LOG.warning(
-                        "Entity missing for %s: %s or for grid: %s (%s).",
-                        model,
-                        model_key,
-                        grid_entity_key,
-                        e,
-                    )
-                model_ctrs[model] += 1
+        # model_ctrs.setdefault(model, 0)
+        # model_key = self.scenario.generate_model_key(
+        #     self, model.lower(), info["bus"], model_ctr
+        # )
+        attrs = info["outputs"][:2]
+        if (
+            self.sim_params["pv_send_p_possible_max_mw_to_grid"]
+            and info["alias"].lower() == "pv"
+        ):
+            attrs.append(("p_possible_max_mw", "max_p_mw"))
+        grid_entity_key = None
+        try:
+            grid_entity_key = self.get_grid_entity(info["type"], info["bus"])
+            self.connect_entities(model_key, grid_entity_key, attrs)
+        except ScenarioError as e:
+            LOG.warning(
+                "Encountered scenario error while connecting %s"
+                "to grid entity %s: %s",
+                model_key,
+                grid_entity_key,
+                e,
+            )
+        except KeyError as e:
+            LOG.warning(
+                "Entity missing for %s: %s or for grid: %s (%s).",
+                info["alias"],
+                model_key,
+                grid_entity_key,
+                e,
+            )
+        except ValueError as e:
+            LOG.warning(
+                "Entity missing for %s: %s or for grid: %s (%s).",
+                info["alias"],
+                model_key,
+                grid_entity_key,
+                e,
+            )
+        # model_ctrs[model] += 1
 
     def connect_to_db(self):
         """Connect the models to db."""
@@ -348,7 +405,6 @@ class PysimmodsModule(UpgradeModule):
 
         model_ctrs = dict()
         for model, info in self.models.items():
-
             for bus, entities in self.sim_params[mapping_key].items():
                 for name, _ in entities:
                     if model != name:
@@ -418,7 +474,7 @@ class PysimmodsModule(UpgradeModule):
 
         wpmapping = dict()
         for models in self.sim_params["mapping"].values():
-            for (model, _) in models:
+            for model, _ in models:
                 wpmapping.setdefault(model, dict())
                 wpmapping[model].setdefault(wprovider, list())
                 wpmapping[model][wprovider].append(0)
@@ -512,13 +568,7 @@ class PysimmodsModule(UpgradeModule):
             self.scenario.actuators.append(actuator)
             LOG.debug("Created actuator entry %s.", actuator)
 
-    def download(
-        self,
-        data_path: str,
-        tmp_path: str,
-        load_if_necessary: bool,
-        force: bool,
-    ):
+    def download(self, data_path: str, tmp_path: str, force: bool):
         # No downloads
         pass
 
@@ -536,12 +586,7 @@ class PysimmodsModule(UpgradeModule):
 
 
 def create_default_mapping():
-    return {
-        3: [("PV", 6)],
-        4: [("PV", 2)],
-        8: [("PV", 2)],
-        11: [("PV", 3)],
-    }
+    return {3: [("PV", 6)], 4: [("PV", 2)], 8: [("PV", 2)], 11: [("PV", 3)]}
 
 
 def get_presets(model, p_peak_mw, **kwargs):
@@ -565,18 +610,24 @@ def get_presets(model, p_peak_mw, **kwargs):
         return diesel_presets(p_max_kw=p_peak_mw * 1e3)
     elif model == "Biogas":
         return biogas_preset(pn_max_kw=p_peak_mw * 1e3)
+    elif model == "Wind":
+        params, inits = wind_presets(pn_max_kw=p_peak_mw * 1e3)
+        params["sn_mva"] = params["inverter"]["sn_kva"] * 1e-3
+        return params, inits
     else:
         raise ValueError(f"Model {model} is unknown.")
 
 
 def create_actuator(
-    full_id: str, attr: str, vmin: float, vmax: float, dtype: str = "np.double"
+    full_id: str,
+    attr: str,
+    vmin: float,
+    vmax: float,
+    dtype: str = "np.float32",
 ):
     return {
-        "actuator_id": f"{full_id}.{attr}",
-        "action_space": (
-            f"Box(low={vmin}, high={vmax}, shape=(1,), dtype={dtype})"
-        ),
+        "uid": f"{full_id}.{attr}",
+        "space": (f"Box(low={vmin}, high={vmax}, shape=(), dtype={dtype})"),
     }
 
 
@@ -585,13 +636,11 @@ def create_sensor(
     attr: str,
     vmin: float = -10000,
     vmax: float = 10000,
-    dtype: str = "np.double",
+    dtype: str = "np.float32",
 ):
     return {
-        "sensor_id": f"{full_id}.{attr}",
-        "observation_space": (
-            f"Box(low={vmin}, high={vmax}, shape=(1,), dtype={dtype})"
-        ),
+        "uid": f"{full_id}.{attr}",
+        "space": (f"Box(low={vmin}, high={vmax}, shape=(), dtype={dtype})"),
     }
 
 
