@@ -2,18 +2,20 @@
 of the pysimmods package.
 
 """
+
 import json
 import logging
 import pprint
-from datetime import datetime, timedelta, timezone
-from typing import Any, Dict
 import warnings
+from datetime import datetime, timedelta, timezone
+from typing import Any, Dict, List, Optional
 
-import mosaik_api
+import mosaik_api_v3
 import numpy as np
 from midas.util.dict_util import strtobool
 from midas.util.logging import set_and_init_logger
 from midas.util.runtime_config import RuntimeConfig
+
 from pysimmods.model.model import Model
 from pysimmods.mosaik import LOG
 from pysimmods.util.date_util import GER
@@ -23,21 +25,28 @@ from .meta import META, MODELS
 VLOG = logging.getLogger("pysimmods.mosaik.verbose")
 
 
-class PysimmodsSimulator(mosaik_api.Simulator):
+class PysimmodsSimulator(mosaik_api_v3.Simulator):
     """The Pysimmods simulator."""
 
     def __init__(self):
         super().__init__(META)
         self.sid: str
+        self.step_size: int
+        self.now_dt: datetime
+        self.key_value_logs: bool
+
         self.models: Dict[str, Model] = {}
         self.num_models: Dict[str, int] = {}
 
-        self.step_size: int
-        self.now_dt: datetime
-        self.percent_factor: float = 1.0
-        self.key_value_logs: bool
-
-    def init(self, sid, **sim_params):
+    def init(
+        self,
+        sid: str,
+        start_date: str,
+        step_size: int = 900,
+        key_value_logs: Optional[bool] = None,
+        time_resolution: float = 1.0,
+        **kwargs,  # for compatibility
+    ):
         """Called exactly ones after the simulator has been started.
 
         Parameters
@@ -48,6 +57,11 @@ class PysimmodsSimulator(mosaik_api.Simulator):
             The start date as UTC ISO 8601 date string.
         step_size : int, optional
             Step size for this simulator. Defaults to 900.
+        key_value_logs: bool, optional
+            If set to True, a different, json-based logging format will
+            be used. Default is False.
+        time_resolution: float, optional
+            Something new from mosaik 3.
 
         Returns
         -------
@@ -56,43 +70,41 @@ class PysimmodsSimulator(mosaik_api.Simulator):
 
         """
         self.sid = sid
-        if "step_size" not in sim_params:
-            LOG.debug(
-                "Param *step_size* not provided. "
-                "Using default step size of 900."
-            )
-        self.step_size = sim_params.get("step_size", 900)
-        self.now_dt = datetime.strptime(
-            sim_params["start_date"], GER
-        ).astimezone(timezone.utc)
-
-        use_decimal_percent = sim_params.get("use_decimal_percent", False)
-        if not isinstance(use_decimal_percent, bool):
-            try:
-                use_decimal_percent = strtobool(use_decimal_percent)
-            except ValueError:
-                use_decimal_percent = False
-
-        if use_decimal_percent:
-            self.percent_factor = 0.01
-        else:
-            self.percent_factor = 1.0
-
-        self.key_value_logs = sim_params.get(
-            "key_value_logs", RuntimeConfig().misc.get("key_value_logs", False)
+        self.step_size = step_size
+        self.now_dt = datetime.strptime(start_date, GER).astimezone(
+            timezone.utc
         )
+
+        self.key_value_logs = key_value_logs
+        if self.key_value_logs is None:
+            self.key_value_logs = RuntimeConfig().misc.get(
+                "key_value_logs", False
+            )
+
         return self.meta
 
-    def create(self, num, model, **model_params):
+    def create(
+        self,
+        num: int,
+        model: str,
+        params: Dict[str, Any],
+        inits: Dict[str, Any],
+    ):
         """Initialize the simulation model instance (entity).
 
         Parameters
         ----------
-        num : int
+        num: int
             The number of models to create.
-        model : str
+        model: str
             The name of the models to create. Must be present inside
             the simulator's meta.
+        params: Dict[str, Any]
+            The dictionary with parameters for the model to be created.
+            Must be provided but can be empty.
+        inits: Dict[str, Any]
+            The dictionary with initial values for the state of the
+            model to be created. Must be provided but can be empty.
 
         Returns
         -------
@@ -100,13 +112,10 @@ class PysimmodsSimulator(mosaik_api.Simulator):
             A list with information on the created entity.
 
         """
-        entities = list()
-        params = model_params["params"]
-        inits = model_params["inits"]
+        entities = []
         self.num_models.setdefault(model, 0)
 
         for _ in range(num):
-
             eid = f"{model}-{self.num_models[model]}"
             self.models[eid] = MODELS[model](params, inits)
             self.num_models[model] += 1
@@ -114,7 +123,12 @@ class PysimmodsSimulator(mosaik_api.Simulator):
 
         return entities
 
-    def step(self, time, inputs, max_advance=0):
+    def step(
+        self,
+        time: int,
+        inputs: Dict[str, Dict[str, Dict[str, Any]]],
+        max_advance: int = 0,
+    ):
         """Perform a simulation step.
 
         Parameters
@@ -124,6 +138,8 @@ class PysimmodsSimulator(mosaik_api.Simulator):
             simulation start.
         inputs : dict
             A *dict* containing inputs for entities of this simulator.
+        max_advance: int,
+            Required for mosaik>=3 but not explicitly supported yet.
 
         Returns
         -------
@@ -143,6 +159,8 @@ class PysimmodsSimulator(mosaik_api.Simulator):
         # Set inputs from other simulators
         for eid, attrs in inputs.items():
             for attr, src_ids in attrs.items():
+                if self.key_value_logs:
+                    self._log_input(eid, attr, src_ids)
 
                 # Use time information from time generator
                 if attr == "local_time":
@@ -161,7 +179,9 @@ class PysimmodsSimulator(mosaik_api.Simulator):
 
         return time + self.step_size
 
-    def get_data(self, outputs):
+    def get_data(
+        self, outputs: Dict[str, List[str]]
+    ) -> Dict[str, Dict[str, Any]]:
         """Return the requested output (if feasible).
 
         Parameters
@@ -176,21 +196,14 @@ class PysimmodsSimulator(mosaik_api.Simulator):
 
         """
 
-        data = dict()
+        data = {}
         for eid, attrs in outputs.items():
-            log_msg = {
-                "id": f"{self.sid}_{eid}",
-                "name": eid,
-                "type": eid.split("-")[0],
-            }
-
             for attr in attrs:
                 value = self._get_remaining_attrs(eid, attr)
                 data.setdefault(eid, dict())[attr] = value
-                log_msg[attr] = value
 
-            if self.key_value_logs:
-                LOG.info(json.dumps(log_msg))
+                if self.key_value_logs:
+                    self._log_output(eid, attr, value)
 
         if not self.key_value_logs:
             LOG.debug("Gathered outputs: %s.", pprint.pformat(data))
@@ -207,6 +220,43 @@ class PysimmodsSimulator(mosaik_api.Simulator):
             model.set_step_size(self.step_size)
             model.set_now_dt(self.now_dt)
 
+    def _log_input(self, eid: str, attr: str, src_ids: Dict[str, Any]):
+        for src_id, val in src_ids.items():
+            if isinstance(val, np.ndarray):
+                val = val.tolist()
+            elif isinstance(val, np.integer):
+                val = int(val)
+            elif isinstance(val, (np.float32, np.floating)):
+                val = float(val)
+
+            LOG.debug(
+                json.dumps(
+                    {
+                        "id": f"{self.sid}_{eid}",
+                        "name": eid,
+                        "model": eid.split("-")[0],
+                        "type": "input",
+                        "attribute": attr,
+                        "source": src_id,
+                        "value": val,
+                    }
+                )
+            )
+
+    def _log_output(self, eid: str, attr: str, value: Any):
+        LOG.debug(
+            json.dumps(
+                {
+                    "id": f"{self.sid}_{eid}",
+                    "name": eid,
+                    "model": eid.split("-")[0],
+                    "type": "output",
+                    "attribute": attr,
+                    "value": value,
+                }
+            )
+        )
+
     def _set_attr_local_time(self, eid: str, src_ids: Dict[str, Any]) -> bool:
         for val in src_ids.values():
             self.models[eid].set_now_dt(val)
@@ -233,12 +283,13 @@ class PysimmodsSimulator(mosaik_api.Simulator):
 
         return float(attr_sum)
 
-    def _set_percent_power(self, eid: str, attr_sum: float):
-        attr_sum /= self.percent_factor
-        attr_sum = self.models[eid].get_pn_min_kw() + attr_sum * (
-            self.models[eid].get_pn_max_kw() - self.models[eid].get_pn_min_kw()
-        )
-        self.models[eid].set_p_kw(attr_sum)
+    # def _set_percent_power(self, eid: str, attr_sum: float):
+    #     attr_sum /= self.percent_factor
+    #     attr_sum = self.models[eid].get_pn_min_kw() + attr_sum * (
+    #         self.models[eid].get_pn_max_kw() - self.models[eid].
+    # get_pn_min_kw()
+    #     )
+    #     self.models[eid].set_p_kw(attr_sum)
 
     def _set_remaining_attrs(self, eid: str, attr: str, attr_sum: float):
         # Apply corrections
@@ -249,10 +300,11 @@ class PysimmodsSimulator(mosaik_api.Simulator):
         # Set the inputs
         if attr == "set_percent":
             warnings.warn(
-                "Using set_percent is deprecated and will be removed with pysimmods>=0.9.0",
+                "Using set_percent is deprecated and will be ignored. Use "
+                "p_set_kw or p_set_mw instead",
                 UserWarning,
             )
-            self._set_percent_power(eid, attr_sum)
+
         elif attr == "p_set_kw":
             self.models[eid].set_p_kw(attr_sum)
         elif attr == "q_set_kvar":
@@ -264,6 +316,8 @@ class PysimmodsSimulator(mosaik_api.Simulator):
         # Apply correction of the attr if necessary
         if attr in ("p_mw", "p_th_mw", "q_mvar"):
             true_attr = attr.replace("m", "k")
+        elif attr == "p_possible_max_mw":
+            true_attr = "p_possible_max_kw"
         else:
             true_attr = attr
 
@@ -275,7 +329,7 @@ class PysimmodsSimulator(mosaik_api.Simulator):
             value = getattr(self.models[eid].state, true_attr)
 
         # Apply correction of the value if necessary
-        if attr in ("p_mw", "p_th_mw", "q_mvar"):
+        if attr in ("p_mw", "p_th_mw", "q_mvar", "p_possible_max_mw"):
             value *= 1e-3
 
         return value
@@ -286,4 +340,4 @@ if __name__ == "__main__":
         0, "pysimmods-logfile", "pysimmods-mosaik.log", replace=True
     )
     LOG.info("Starting mosaik simulation...")
-    mosaik_api.start_simulation(PysimmodsSimulator())
+    mosaik_api_v3.start_simulation(PysimmodsSimulator())
