@@ -2,13 +2,18 @@
 :class:`.ForecastModel` and :class:`.FlexibilityModel`.
 
 """
+
+from __future__ import annotations
+
 import logging
 from datetime import datetime, timedelta, timezone
-from typing import Optional
+from io import StringIO
+from typing import Dict, Optional
 
 import numpy as np
 import pandas as pd
-from pysimmods.util.date_util import GER
+
+from pysimmods.util.date_util import GER, TZ
 
 LOG = logging.getLogger(__name__)
 
@@ -37,6 +42,7 @@ class Schedule:
         q_name: str = "q_kvar",
         p_set_name: Optional[str] = None,
         q_set_name: Optional[str] = None,
+        store_min_and_max: bool = False,
     ):
         self._data: pd.DataFrame
 
@@ -48,6 +54,12 @@ class Schedule:
         self.q_name = q_name
         self.p_set_name = p_set_name
         self.q_set_name = q_set_name
+        self.p_min_name = "p_min"
+        self.p_max_name = "p_max"
+        self.q_min_name = "q_min"
+        self.q_max_name = "q_max"
+
+        self.store_min_and_max = store_min_and_max
 
     def init(self):
         """Initialize the schedule data frame.
@@ -66,6 +78,16 @@ class Schedule:
         )
         columns = [self.p_name, self.q_name]
 
+        if self.store_min_and_max:
+            columns.extend(
+                [
+                    self.p_min_name,
+                    self.p_max_name,
+                    self.q_min_name,
+                    self.q_max_name,
+                ]
+            )
+
         if self.p_set_name is None:
             unit = self.p_name.rsplit("_", 1)[1]
             self.p_set_name = f"p_set_{unit}"
@@ -82,7 +104,7 @@ class Schedule:
                 self.now_dt
                 + timedelta(hours=self.horizon_hours)
                 - timedelta(seconds=self.step_size),
-                freq=f"{self.step_size}S",
+                freq=f"{self.step_size}s",
             )
         else:
             index = None
@@ -119,7 +141,12 @@ class Schedule:
                 break
 
         self._data.update(other.loc[:index])
-        self._data = pd.concat([self._data, other.loc[index:]])
+        other = other.loc[index:]
+        # https://stackoverflow.com/questions/77254777/alternative-to-concat-
+        # of-empty-dataframe-now-that-it-is-being-deprecated
+        self._data = pd.concat(
+            [df for df in [self._data, other] if not df.empty]
+        )
         self._data = self._data[~self._data.index.duplicated()]
         self._data.index = pd.to_datetime(self._data.index, utc=True)
 
@@ -130,13 +157,22 @@ class Schedule:
         q_set: Optional[float],
         p_val: float,
         q_val: float,
+        p_min: float = 0,
+        p_max: float = 0,
+        q_min: float = 0,
+        q_max: float = 0,
     ):
-
         if index in self._data.index:
-            self._data.loc[index][self.p_set_name] = p_set
-            self._data.loc[index][self.q_set_name] = q_set
-            self._data.loc[index][self.p_name] = p_val
-            self._data.loc[index][self.q_name] = q_val
+            self._data.loc[index, self.p_set_name] = p_set
+            self._data.loc[index, self.q_set_name] = q_set
+            self._data.loc[index, self.p_name] = p_val
+            self._data.loc[index, self.q_name] = q_val
+
+            if self.store_min_and_max:
+                self._data.loc[index, self.p_min_name] = p_min
+                self._data.loc[index, self.p_max_name] = p_max
+                self._data.loc[index, self.q_min_name] = q_min
+                self._data.loc[index, self.q_max_name] = q_max
 
         else:
             tmp_df = pd.DataFrame(
@@ -149,11 +185,19 @@ class Schedule:
                 index=[index],
             )
 
+            if self.store_min_and_max:
+                self._data.loc[index, self.p_min_name] = p_min
+                self._data.loc[index, self.p_max_name] = p_max
+                self._data.loc[index, self.q_min_name] = q_min
+                self._data.loc[index, self.q_max_name] = q_max
+
             self._data = pd.concat([self._data, tmp_df])
 
         self._data.sort_index(inplace=True)
         self._data.index = pd.to_datetime(self._data.index)
-        self._data = self._data.fillna(value=np.nan)
+        # self._data = self._data.ffill()
+        # WTF pandas?!?
+        self._data = self._data.astype(np.float64).fillna(value=np.nan)
 
     def update_entry(self, index, col, val):
         if col not in self._data.columns:
@@ -165,9 +209,9 @@ class Schedule:
             self._data = pd.concat(
                 [self._data, pd.DataFrame({col: val}, index=[index])]
             )
-        self._data.loc[index][col] = val
+        self._data.loc[index, col] = val
         self._data.sort_index(inplace=True)
-        self._data = self._data.fillna(value=np.nan)
+        self._data = self._data.infer_objects(copy=False).fillna(value=np.nan)
 
     def has_index(self, index):
         return index in self._data.index
@@ -199,6 +243,15 @@ class Schedule:
                 return True
             elif self.get(now, self.q_name) is None:
                 return True
+            if self.store_min_and_max:
+                if self.get(now, self.p_min_name) is None:
+                    return True
+                if self.get(now, self.p_max_name) is None:
+                    return True
+                if self.get(now, self.q_min_name) is None:
+                    return True
+                if self.get(now, self.q_max_name) is None:
+                    return True
 
             now += timedelta(seconds=self.step_size)
 
@@ -233,6 +286,34 @@ class Schedule:
         else:
             return val
 
+    def to_dict(
+        self,
+        start_dt: Optional[datetime] = None,
+        end_dt: Optional[datetime] = None,
+        keep_datetime: bool = False,
+    ) -> Dict[str, str]:
+        if start_dt is None:
+            start_dt = self._data.index[0]
+
+        if end_dt is None:
+            end_dt = self._data.index[-1]
+
+        partition = self._data.loc[start_dt:end_dt]
+
+        if not keep_datetime:
+            partition.index = partition.index.astype(np.int64)
+
+        return partition.to_dict()
+
+    def from_dict(self, dict_of_schedules: Dict[str, str]) -> Schedule:
+        df = pd.DataFrame(dict_of_schedules)
+        df.index = pd.to_datetime(
+            df.index.astype(np.int64), utc=True, origin="unix"
+        )
+        self.from_dataframe(df)
+
+        return self
+
     def to_json(
         self,
         start_dt: Optional[datetime] = None,
@@ -246,8 +327,8 @@ class Schedule:
 
         return self._data.loc[start_dt:end_dt].to_json()
 
-    def from_json(self, schedule_json):
-        schedule_df = pd.read_json(schedule_json).tz_localize("UTC")
+    def from_json(self, schedule_json) -> Schedule:
+        schedule_df = pd.read_json(StringIO(schedule_json)).tz_localize("UTC")
         self.from_dataframe(schedule_df)
 
     def __call__(self):
@@ -255,7 +336,7 @@ class Schedule:
 
     def from_dataframe(self, dataframe: pd.DataFrame):
         self._data = dataframe
-        self._data.index = pd.to_datetime(self._data.index)
+        self._data.index = pd.to_datetime(self._data.index).tz_convert(TZ)
         if "target" in self._data.columns:
             self.use_absolute_setpoints = False
             self.p_set_name = None
@@ -268,6 +349,18 @@ class Schedule:
                 self.p_set_name = col
             elif "q_set" in col:
                 self.q_set_name = col
+            elif "p_min" in col:
+                self.p_min_name = col
+                self.store_min_and_max = True
+            elif "p_max" in col:
+                self.p_max_name = col
+                self.store_min_and_max = True
+            elif "q_min" in col:
+                self.q_min_name = col
+                self.store_min_and_max = True
+            elif "q_max" in col:
+                self.q_max_name = col
+                self.store_min_and_max = True
             elif "p_" in col:
                 self.p_name = col
             elif "q_" in col:
@@ -281,6 +374,7 @@ class Schedule:
                 / (len(self._data.index) - 1)
             ),
         )
+        return self
 
     def __repr__(self):
         return self._data.__repr__()
@@ -290,3 +384,6 @@ class Schedule:
             return True
 
         return self._data.empty
+
+    def compare(self, other: Schedule):
+        return self._data.compare(other._data)
