@@ -1,7 +1,7 @@
-"""This module contains the state information of the heat demand model.
-"""
+"""This module contains the state information of the heat demand model."""
 
 from datetime import datetime
+
 from pysimmods.util.date_util import GER
 
 
@@ -9,7 +9,6 @@ class HeatDemandState:
     """Captures the state of the heat demand model"""
 
     def __init__(self, inits):
-
         start_date = inits["start_date"]
         if isinstance(start_date, str):
             self.now_dt = datetime.strptime(start_date, GER)
